@@ -1,5 +1,10 @@
 ## ----installGithub, eval=FALSE-------------------------------------------
-#  devtools::install_github('dmlc/xgboost', subdir='R-package')
+#  install.packages("drat", repos="https://cran.rstudio.com")
+#  drat:::addRepo("dmlc")
+#  install.packages("xgboost", repos="http://dmlc.ml/drat/", type = "source")
+
+## ---- eval=FALSE---------------------------------------------------------
+#  install.packages("xgboost")
 
 ## ----libLoading, results='hold', message=F, warning=F--------------------
 require(xgboost)
@@ -22,26 +27,26 @@ class(train$data)[1]
 class(train$label)
 
 ## ----trainingSparse, message=F, warning=F--------------------------------
-bstSparse <- xgboost(data = train$data, label = train$label, max.depth = 2, eta = 1, nthread = 2, nround = 2, objective = "binary:logistic")
+bstSparse <- xgboost(data = train$data, label = train$label, max_depth = 2, eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
 
 ## ----trainingDense, message=F, warning=F---------------------------------
-bstDense <- xgboost(data = as.matrix(train$data), label = train$label, max.depth = 2, eta = 1, nthread = 2, nround = 2, objective = "binary:logistic")
+bstDense <- xgboost(data = as.matrix(train$data), label = train$label, max_depth = 2, eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
 
 ## ----trainingDmatrix, message=F, warning=F-------------------------------
 dtrain <- xgb.DMatrix(data = train$data, label = train$label)
-bstDMatrix <- xgboost(data = dtrain, max.depth = 2, eta = 1, nthread = 2, nround = 2, objective = "binary:logistic")
+bstDMatrix <- xgboost(data = dtrain, max_depth = 2, eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic")
 
 ## ----trainingVerbose0, message=T, warning=F------------------------------
 # verbose = 0, no message
-bst <- xgboost(data = dtrain, max.depth = 2, eta = 1, nthread = 2, nround = 2, objective = "binary:logistic", verbose = 0)
+bst <- xgboost(data = dtrain, max_depth = 2, eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic", verbose = 0)
 
 ## ----trainingVerbose1, message=T, warning=F------------------------------
 # verbose = 1, print evaluation metric
-bst <- xgboost(data = dtrain, max.depth = 2, eta = 1, nthread = 2, nround = 2, objective = "binary:logistic", verbose = 1)
+bst <- xgboost(data = dtrain, max_depth = 2, eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic", verbose = 1)
 
 ## ----trainingVerbose2, message=T, warning=F------------------------------
 # verbose = 2, also print information about tree
-bst <- xgboost(data = dtrain, max.depth = 2, eta = 1, nthread = 2, nround = 2, objective = "binary:logistic", verbose = 2)
+bst <- xgboost(data = dtrain, max_depth = 2, eta = 1, nthread = 2, nrounds = 2, objective = "binary:logistic", verbose = 2)
 
 ## ----predicting, message=F, warning=F------------------------------------
 pred <- predict(bst, test$data)
@@ -67,19 +72,19 @@ dtest <- xgb.DMatrix(data = test$data, label=test$label)
 ## ----watchlist, message=F, warning=F-------------------------------------
 watchlist <- list(train=dtrain, test=dtest)
 
-bst <- xgb.train(data=dtrain, max.depth=2, eta=1, nthread = 2, nround=2, watchlist=watchlist, objective = "binary:logistic")
+bst <- xgb.train(data=dtrain, max_depth=2, eta=1, nthread = 2, nrounds=2, watchlist=watchlist, objective = "binary:logistic")
 
 ## ----watchlist2, message=F, warning=F------------------------------------
-bst <- xgb.train(data=dtrain, max.depth=2, eta=1, nthread = 2, nround=2, watchlist=watchlist, eval.metric = "error", eval.metric = "logloss", objective = "binary:logistic")
+bst <- xgb.train(data=dtrain, max_depth=2, eta=1, nthread = 2, nrounds=2, watchlist=watchlist, eval_metric = "error", eval_metric = "logloss", objective = "binary:logistic")
 
 ## ----linearBoosting, message=F, warning=F--------------------------------
-bst <- xgb.train(data=dtrain, booster = "gblinear", max.depth=2, nthread = 2, nround=2, watchlist=watchlist, eval.metric = "error", eval.metric = "logloss", objective = "binary:logistic")
+bst <- xgb.train(data=dtrain, booster = "gblinear", max_depth=2, nthread = 2, nrounds=2, watchlist=watchlist, eval_metric = "error", eval_metric = "logloss", objective = "binary:logistic")
 
 ## ----DMatrixSave, message=F, warning=F-----------------------------------
 xgb.DMatrix.save(dtrain, "dtrain.buffer")
 # to load it in, simply call xgb.DMatrix
 dtrain2 <- xgb.DMatrix("dtrain.buffer")
-bst <- xgb.train(data=dtrain2, max.depth=2, eta=1, nthread = 2, nround=2, watchlist=watchlist, objective = "binary:logistic")
+bst <- xgb.train(data=dtrain2, max_depth=2, eta=1, nthread = 2, nrounds=2, watchlist=watchlist, objective = "binary:logistic")
 
 ## ----DMatrixDel, include=FALSE-------------------------------------------
 file.remove("dtrain.buffer")
@@ -91,7 +96,7 @@ err <- as.numeric(sum(as.integer(pred > 0.5) != label))/length(label)
 print(paste("test-error=", err))
 
 ## ----dump, message=T, warning=F------------------------------------------
-xgb.dump(bst, with.stats = T)
+xgb.dump(bst, with_stats = T)
 
 ## ----saveModel, message=F, warning=F-------------------------------------
 # save model to binary local file
