@@ -2,19 +2,21 @@
  * Copyright (c) 2015 by Contributors
  * \file c_api.h
  * \author Tianqi Chen
- * \brief C API of XGBoost, used to interfacing with other languages.
+ * \brief C API of XGBoost, used for interfacing to other languages.
  */
 #ifndef XGBOOST_C_API_H_
 #define XGBOOST_C_API_H_
 
 #ifdef __cplusplus
 #define XGB_EXTERN_C extern "C"
+#include <cstdio>
+#else
+#define XGB_EXTERN_C
+#include <stdio.h>
+#include <stdint.h>
 #endif
 
 // XGBoost C API will include APIs in Rabit C API
-XGB_EXTERN_C {
-#include <stdio.h>
-}
 #include <rabit/c_api.h>
 
 #if defined(_MSC_VER) || defined(_WIN32)
@@ -28,16 +30,16 @@ typedef uint64_t bst_ulong;  // NOLINT(*)
 
 
 /*! \brief handle to DMatrix */
-typedef void *DMatrixHandle;
+typedef void *DMatrixHandle;  // NOLINT(*)
 /*! \brief handle to Booster */
-typedef void *BoosterHandle;
+typedef void *BoosterHandle;  // NOLINT(*)
 /*! \brief handle to a data iterator */
-typedef void *DataIterHandle;
+typedef void *DataIterHandle;  // NOLINT(*)
 /*! \brief handle to a internal data holder. */
-typedef void *DataHolderHandle;
+typedef void *DataHolderHandle;  // NOLINT(*)
 
 /*! \brief Mini batch used in XGBoost Data Iteration */
-typedef struct {
+typedef struct {  // NOLINT(*)
   /*! \brief number of rows in the minibatch */
   size_t size;
   /*! \brief row pointer to the rows in the data */
@@ -62,9 +64,9 @@ typedef struct {
 /*!
  * \brief Callback to set the data to handle,
  * \param handle The handle to the callback.
- * \param batch The data content to be setted.
+ * \param batch The data content to be set.
  */
-XGB_EXTERN_C typedef int XGBCallbackSetData(
+XGB_EXTERN_C typedef int XGBCallbackSetData(  // NOLINT(*)
     DataHolderHandle handle, XGBoostBatchCSR batch);
 
 /*!
@@ -78,9 +80,8 @@ XGB_EXTERN_C typedef int XGBCallbackSetData(
  * \param set_function_handle The handle to be passed to set function.
  * \return 0 if we are reaching the end and batch is not returned.
  */
-XGB_EXTERN_C typedef int XGBCallbackDataIterNext(
-    DataIterHandle data_handle,
-    XGBCallbackSetData* set_function,
+XGB_EXTERN_C typedef int XGBCallbackDataIterNext(  // NOLINT(*)
+    DataIterHandle data_handle, XGBCallbackSetData *set_function,
     DataHolderHandle set_function_handle);
 
 /*!
@@ -93,7 +94,7 @@ XGB_EXTERN_C typedef int XGBCallbackDataIterNext(
  *  this function is thread safe and can be called by different thread
  * \return const char* error information
  */
-XGB_DLL const char *XGBGetLastError();
+XGB_DLL const char *XGBGetLastError(void);
 
 /*!
  * \brief load a data matrix
@@ -204,6 +205,20 @@ XGB_DLL int XGDMatrixCreateFromMat(const float *data,
                                    bst_ulong ncol,
                                    float missing,
                                    DMatrixHandle *out);
+/*!
+ * \brief create matrix content from dense matrix
+ * \param data pointer to the data space
+ * \param nrow number of rows
+ * \param ncol number columns
+ * \param missing which value to represent missing value
+ * \param out created dmatrix
+ * \param nthread number of threads (up to maximum cores available, if <=0 use all cores)
+ * \return 0 when success, -1 when failure happens
+ */
+XGB_DLL int XGDMatrixCreateFromMat_omp(const float *data,  // NOLINT
+                                       bst_ulong nrow, bst_ulong ncol,
+                                       float missing, DMatrixHandle *out,
+                                       int nthread);
 /*!
  * \brief create a new dmatrix from sliced content of existing matrix
  * \param handle instance of data matrix to be sliced
@@ -220,7 +235,7 @@ XGB_DLL int XGDMatrixSliceDMatrix(DMatrixHandle handle,
  * \brief free space in data matrix
  * \return 0 when success, -1 when failure happens
  */
-XGB_DLL int XGDMatrixFree(void *handle);
+XGB_DLL int XGDMatrixFree(DMatrixHandle handle);
 /*!
  * \brief load a data matrix into binary file
  * \param handle a instance of data matrix
@@ -382,6 +397,7 @@ XGB_DLL int XGBoosterEvalOneIter(BoosterHandle handle,
  *          0:normal prediction
  *          1:output margin instead of transformed value
  *          2:output leaf index of trees instead of leaf value, note leaf index is unique per tree
+ *          4:output feature contributions to individual predictions
  * \param ntree_limit limit number of trees used for prediction, this is only valid for boosted trees
  *    when the parameter is set to 0, we will use all the trees
  * \param out_len used to store length of returning result
