@@ -333,7 +333,8 @@ void CLIPredict(const CLIParam& param) {
       dmlc::Stream::Create(param.name_pred.c_str(), "w"));
   dmlc::ostream os(fo.get());
   for (bst_float p : preds.HostVector()) {
-    os << p << '\n';
+    os << std::setprecision(std::numeric_limits<bst_float>::max_digits10 + 2)
+       << p << '\n';
   }
   // force flush before fo destruct.
   os.set_stream(nullptr);
