@@ -10,24 +10,18 @@ namespace xgboost {
 namespace data {
 
 void SimpleCSRSource::Clear() {
-  row_data_.clear();
-  row_ptr_.resize(1);
-  row_ptr_[0] = 0;
+  page_.Clear();
   this->info.Clear();
 }
 
 void SimpleCSRSource::CopyFrom(DMatrix* src) {
   this->Clear();
   this->info = src->Info();
-  dmlc::DataIter<RowBatch>* iter = src->RowIterator();
+  auto iter = src->RowIterator();
   iter->BeforeFirst();
   while (iter->Next()) {
-    const RowBatch &batch = iter->Value();
-    for (size_t i = 0; i < batch.size; ++i) {
-      RowBatch::Inst inst = batch[i];
-      row_data_.insert(row_data_.end(), inst.data, inst.data + inst.length);
-      row_ptr_.push_back(row_ptr_.back() + inst.length);
-    }
+    const auto &batch = iter->Value();
+    page_.Push(batch);
   }
 }
 
@@ -53,16 +47,16 @@ void SimpleCSRSource::CopyFrom(dmlc::Parser<uint32_t>* parser) {
     for (size_t i = batch.offset[0]; i < batch.offset[batch.size]; ++i) {
       uint32_t index = batch.index[i];
       bst_float fvalue = batch.value == nullptr ? 1.0f : batch.value[i];
-      row_data_.emplace_back(index, fvalue);
+      page_.data.emplace_back(index, fvalue);
       this->info.num_col_ = std::max(this->info.num_col_,
                                     static_cast<uint64_t>(index + 1));
     }
-    size_t top = row_ptr_.size();
+    size_t top = page_.offset.size();
     for (size_t i = 0; i < batch.size; ++i) {
-      row_ptr_.push_back(row_ptr_[top - 1] + batch.offset[i + 1] - batch.offset[0]);
+      page_.offset.push_back(page_.offset[top - 1] + batch.offset[i + 1] - batch.offset[0]);
     }
   }
-  this->info.num_nonzero_ = static_cast<uint64_t>(row_data_.size());
+  this->info.num_nonzero_ = static_cast<uint64_t>(page_.data.size());
 }
 
 void SimpleCSRSource::LoadBinary(dmlc::Stream* fi) {
@@ -70,16 +64,16 @@ void SimpleCSRSource::LoadBinary(dmlc::Stream* fi) {
   CHECK(fi->Read(&tmagic, sizeof(tmagic)) == sizeof(tmagic)) << "invalid input file format";
   CHECK_EQ(tmagic, kMagic) << "invalid format, magic number mismatch";
   info.LoadBinary(fi);
-  fi->Read(&row_ptr_);
-  fi->Read(&row_data_);
+  fi->Read(&page_.offset);
+  fi->Read(&page_.data);
 }
 
 void SimpleCSRSource::SaveBinary(dmlc::Stream* fo) const {
   int tmagic = kMagic;
   fo->Write(&tmagic, sizeof(tmagic));
   info.SaveBinary(fo);
-  fo->Write(row_ptr_);
-  fo->Write(row_data_);
+  fo->Write(page_.offset);
+  fo->Write(page_.data);
 }
 
 void SimpleCSRSource::BeforeFirst() {
@@ -89,15 +83,11 @@ void SimpleCSRSource::BeforeFirst() {
 bool SimpleCSRSource::Next() {
   if (!at_first_) return false;
   at_first_ = false;
-  batch_.size = row_ptr_.size() - 1;
-  batch_.base_rowid = 0;
-  batch_.ind_ptr = dmlc::BeginPtr(row_ptr_);
-  batch_.data_ptr = dmlc::BeginPtr(row_data_);
   return true;
 }
 
-const RowBatch& SimpleCSRSource::Value() const {
-  return batch_;
+const SparsePage& SimpleCSRSource::Value() const {
+  return page_;
 }
 
 }  // namespace data
