@@ -29,8 +29,8 @@ SparsePageDMatrix::ColPageIter::ColPageIter(
     dmlc::SeekStream* fi = files_[i].get();
     std::string format;
     CHECK(fi->Read(&format)) << "Invalid page format";
-    formats_[i].reset(SparsePage::Format::Create(format));
-    SparsePage::Format* fmt = formats_[i].get();
+    formats_[i].reset(SparsePageFormat::Create(format));
+    SparsePageFormat* fmt = formats_[i].get();
     size_t fbegin = fi->Tell();
     prefetchers_[i].reset(new dmlc::ThreadedIter<SparsePage>(4));
     prefetchers_[i]->Init([this, fi, fmt] (SparsePage** dptr) {
@@ -61,15 +61,6 @@ bool SparsePageDMatrix::ColPageIter::Next() {
     prefetchers_[(clock_ptr_ + n - 1) % n]->Recycle(&page_);
   }
   if (prefetchers_[clock_ptr_]->Next(&page_)) {
-    out_.col_index = dmlc::BeginPtr(index_set_);
-    col_data_.resize(page_->offset.size() - 1, SparseBatch::Inst(nullptr, 0));
-    for (size_t i = 0; i < col_data_.size(); ++i) {
-      col_data_[i] = SparseBatch::Inst
-          (dmlc::BeginPtr(page_->data) + page_->offset[i],
-           static_cast<bst_uint>(page_->offset[i + 1] - page_->offset[i]));
-    }
-    out_.col_data = dmlc::BeginPtr(col_data_);
-    out_.size = col_data_.size();
     // advance clock
     clock_ptr_ = (clock_ptr_ + 1) % prefetchers_.size();
     return true;
@@ -85,40 +76,22 @@ void SparsePageDMatrix::ColPageIter::BeforeFirst() {
   }
 }
 
-void SparsePageDMatrix::ColPageIter::Init(const std::vector<bst_uint>& index_set,
-                                          bool load_all) {
+void SparsePageDMatrix::ColPageIter::Init(
+    const std::vector<bst_uint>& index_set) {
   set_index_set_ = index_set;
-  set_load_all_ = load_all;
+  set_load_all_ = true;
   std::sort(set_index_set_.begin(), set_index_set_.end());
   this->BeforeFirst();
 }
 
-dmlc::DataIter<ColBatch>* SparsePageDMatrix::ColIterator() {
+  dmlc::DataIter<SparsePage>* SparsePageDMatrix::ColIterator() {
   CHECK(col_iter_ != nullptr);
   std::vector<bst_uint> col_index;
-  size_t ncol = this->Info().num_col_;
-  for (size_t i = 0; i < ncol; ++i) {
-    col_index.push_back(static_cast<bst_uint>(i));
-  }
-  col_iter_->Init(col_index, true);
+  std::iota(col_index.begin(), col_index.end(), bst_uint(0));
+  col_iter_->Init(col_index);
   return col_iter_.get();
 }
 
-dmlc::DataIter<ColBatch>* SparsePageDMatrix::
-ColIterator(const std::vector<bst_uint>& fset) {
-  CHECK(col_iter_ != nullptr);
-  std::vector<bst_uint> col_index;
-  size_t ncol = this->Info().num_col_;
-  for (auto fidx : fset) {
-    if (fidx < ncol) {
-      col_index.push_back(fidx);
-    }
-  }
-  col_iter_->Init(col_index, false);
-  return col_iter_.get();
-}
-
-
 bool SparsePageDMatrix::TryInitColData(bool sorted) {
   // load meta data.
   std::vector<std::string> cache_shards = common::Split(cache_info_, ':');
@@ -145,9 +118,8 @@ bool SparsePageDMatrix::TryInitColData(bool sorted) {
   return true;
 }
 
-void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
-                                      float pkeep,
-                                      size_t max_row_perbatch, bool sorted) {
+void SparsePageDMatrix::InitColAccess(
+  size_t max_row_perbatch, bool sorted) {
   if (HaveColAccess(sorted)) return;
   if (TryInitColData(sorted)) return;
   const MetaInfo& info = this->Info();
@@ -157,11 +129,9 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
   buffered_rowset_.Clear();
   col_size_.resize(info.num_col_);
   std::fill(col_size_.begin(), col_size_.end(), 0);
-  dmlc::DataIter<RowBatch>* iter = this->RowIterator();
-  std::bernoulli_distribution coin_flip(pkeep);
+  auto iter = this->RowIterator();
   size_t batch_ptr = 0, batch_top = 0;
   SparsePage tmp;
-  auto& rnd = common::GlobalRandom();
 
   // function to create the page.
   auto make_col_batch = [&] (
@@ -169,9 +139,9 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
       size_t begin,
       SparsePage *pcol) {
     pcol->Clear();
-    pcol->min_index = buffered_rowset_[begin];
+    pcol->base_rowid = buffered_rowset_[begin];
     const int nthread = std::max(omp_get_max_threads(), std::max(omp_get_num_procs() / 2 - 1, 1));
-    common::ParallelGroupBuilder<SparseBatch::Entry>
+    common::ParallelGroupBuilder<Entry>
     builder(&pcol->offset, &pcol->data);
     builder.InitBudget(info.num_col_, nthread);
     bst_omp_uint ndata = static_cast<bst_uint>(prow.Size());
@@ -179,10 +149,8 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
     for (bst_omp_uint i = 0; i < ndata; ++i) {
       int tid = omp_get_thread_num();
       for (size_t j = prow.offset[i]; j < prow.offset[i+1]; ++j) {
-        const SparseBatch::Entry &e = prow.data[j];
-        if (enabled[e.index]) {
-          builder.AddBudget(e.index, tid);
-        }
+        const  auto e = prow.data[j];
+        builder.AddBudget(e.index, tid);
       }
     }
     builder.InitStorage();
@@ -190,9 +158,9 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
     for (bst_omp_uint i = 0; i < ndata; ++i) {
       int tid = omp_get_thread_num();
       for (size_t j = prow.offset[i]; j < prow.offset[i+1]; ++j) {
-        const SparseBatch::Entry &e = prow.data[j];
+        const Entry &e = prow.data[j];
         builder.Push(e.index,
-                     SparseBatch::Entry(buffered_rowset_[i + begin], e.fvalue),
+                     Entry(buffered_rowset_[i + begin], e.fvalue),
                      tid);
       }
     }
@@ -205,7 +173,7 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
         if (pcol->offset[i] < pcol->offset[i + 1]) {
           std::sort(dmlc::BeginPtr(pcol->data) + pcol->offset[i],
             dmlc::BeginPtr(pcol->data) + pcol->offset[i + 1],
-            SparseBatch::Entry::CmpValue);
+            Entry::CmpValue);
         }
       }
     }
@@ -217,14 +185,12 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
 
     while (true) {
       if (batch_ptr != batch_top) {
-        const RowBatch& batch = iter->Value();
-        CHECK_EQ(batch_top, batch.size);
+         auto batch = iter->Value();
+        CHECK_EQ(batch_top, batch.Size());
         for (size_t i = batch_ptr; i < batch_top; ++i) {
           auto ridx = static_cast<bst_uint>(batch.base_rowid + i);
-          if (pkeep == 1.0f || coin_flip(rnd)) {
-            buffered_rowset_.PushBack(ridx);
-            tmp.Push(batch[i]);
-          }
+          buffered_rowset_.PushBack(ridx);
+          tmp.Push(batch[i]);
 
           if (tmp.Size() >= max_row_perbatch ||
               tmp.MemCostBytes() >= kPageSize) {
@@ -237,7 +203,7 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
       }
       if (!iter->Next()) break;
       batch_ptr = 0;
-      batch_top = iter->Value().size;
+      batch_top = iter->Value().Size();
     }
 
     if (tmp.Size() != 0) {
@@ -252,11 +218,11 @@ void SparsePageDMatrix::InitColAccess(const std::vector<bool>& enabled,
   std::vector<std::string> name_shards, format_shards;
   for (const std::string& prefix : cache_shards) {
     name_shards.push_back(prefix + ".col.page");
-    format_shards.push_back(SparsePage::Format::DecideFormat(prefix).second);
+    format_shards.push_back(SparsePageFormat::DecideFormat(prefix).second);
   }
 
   {
-    SparsePage::Writer writer(name_shards, format_shards, 6);
+    SparsePageWriter writer(name_shards, format_shards, 6);
     std::shared_ptr<SparsePage> page;
     writer.Alloc(&page); page->Clear();
 
